import contextlib
import unittest.case
from textwrap import dedent

from pydantic import BaseConfig
from pydantic.fields import ModelField

from fastorm import FastORM
from fastorm.classes import FieldInfo, FieldItem

# noinspection PyUnresolvedReferences
_subtest_msg_sentinel = unittest.case._subtest_msg_sentinel


class VerboseTestCase(unittest.TestCase):
    show_real_diffs_in_pycharm_instead_of_having_subtests = True

    def subTest(self, msg=_subtest_msg_sentinel, **params):
        if not VerboseTestCase.show_real_diffs_in_pycharm_instead_of_having_subtests:
            return super().subTest(msg=msg, **params)
        else:
            @contextlib.contextmanager
            def subTestNoOP(msg=msg, **params):
                yield
            # end def
            return subTestNoOP(msg=msg, **params)
        # end if
    # end def

    def setUp(self) -> None:
        self.maxDiff = None
    # end def
# end def


class GetFieldsReferencesSimpleTest(VerboseTestCase):
    def test_simple_double_key_no_recursive(self):

        class OtherTable(FastORM):
            _table_name = 'other_table'
            _primary_keys = ['id_part_1', 'id_part_2']
            id_part_1: int
            id_part_2: str

        class ActualTable(FastORM):
            _table_name = 'actual_table'
            _primary_keys = ['cool_reference']
            cool_reference: OtherTable


        with self.subTest():
            expected = dedent("""
            {'cool_reference': FieldInfo(is_primary_key=True, types=[FieldItem(field='cool_reference', type_=ModelField(name='cool_reference', type=OtherTable, required=True))])}
            """).strip()
            refs = ActualTable.get_fields_typehints(flatten_table_references=False)
            self.assertEqual(expected, repr(refs))
        # end with
        with self.subTest():
            expected = dedent("""
            {'cool_reference__id_part_1': FieldInfo(is_primary_key=True, types=[FieldItem(field='cool_reference', type_=ModelField(name='cool_reference', type=OtherTable, required=True)), FieldItem(field='id_part_1', type_=ModelField(name='id_part_1', type=int, required=True))]), 'cool_reference__id_part_2': FieldInfo(is_primary_key=True, types=[FieldItem(field='cool_reference', type_=ModelField(name='cool_reference', type=OtherTable, required=True)), FieldItem(field='id_part_2', type_=ModelField(name='id_part_2', type=str, required=True))])}            """).strip()
            refs = ActualTable.get_fields_typehints(flatten_table_references=True)
            self.assertEqual(expected, repr(refs))
        # end with
    # end def
# end class

if __name__ == '__main__':
    unittest.main()



import unittest
from fastorm import FastORM


class OtherTable(FastORM):
    _table_name = 'other_table'
    _primary_keys = ['id_part_1', 'id_part_2']
    _ignored_fields = []

    id_part_1: int
    id_part_2: str
# end class


class ActualTable(FastORM):
    _table_name = 'actual_table'
    _primary_keys = ['cool_reference']
    _ignored_fields = []
    cool_reference: OtherTable
# end class


class ThirdTable(FastORM):
    _table_name = 'third_table'
    _primary_keys = ['the_reference_has_been_doubled']
    _ignored_fields = []
    the_reference_has_been_doubled: ActualTable
# end class


class MyTestCase(unittest.TestCase):
    def test_flattened(self):
        type_hints = ActualTable.get_fields_typehints(flatten_table_references=True)
        """
        {
            'cool_reference__id_part_1': ModelField(name='id_part_1', type=int, required=True),
            'cool_reference__id_part_2': ModelField(name='id_part_2', type=str, required=True)
        }
        """
        print(type_hints)
        self.assertListEqual(list(type_hints.keys()), ['cool_reference__id_part_1', 'cool_reference__id_part_2'])
        self.assertEqual(type_hints['cool_reference__id_part_1'].types[-1].type_.type_, int)
        self.assertEqual(type_hints['cool_reference__id_part_2'].types[-1].type_.type_, str)
    # end def

    def test_not_fattened(self):
        type_hints = ActualTable.get_fields_typehints(flatten_table_references=False)
        """
        {
            'cool_reference': ModelField(name='cool_reference', type=OtherTable, required=True)
        }
        """
        self.assertListEqual(list(type_hints.keys()), ['cool_reference'])
        self.assertEqual(type_hints['cool_reference'].types[0].type_.type_, OtherTable)
    # end def

    @unittest.skip('Currently out of scope.')
    def test_flattened_double_level(self):
        type_hints = ThirdTable.get_fields_typehints(flatten_table_references=True)
        """
        {
            'the_reference_has_been_doubled__cool_reference__id_part_1': ModelField(name='id_part_1', type=int, required=True),
            'the_reference_has_been_doubled__cool_reference__id_part_2': ModelField(name='id_part_2', type=str, required=True)
        }
        """
        print(type_hints)
        self.assertListEqual(list(type_hints.keys()), ['the_reference_has_been_doubled__cool_reference__id_part_1', 'the_reference_has_been_doubled__cool_reference__id_part_2'])
        self.assertEqual(type_hints['the_reference_has_been_doubled__cool_reference__id_part_1'].type_, int)
        self.assertEqual(type_hints['the_reference_has_been_doubled__cool_reference__id_part_2'].type_, str)
    # end def
# end class

if __name__ == '__main__':
    unittest.main()
# end if
